
# this script does the following:
# 1) runs the three Bayesian model types (linear, segmented, flat) using rstan 
	# note that running the models requires a total of 6 .stan files to be referenced. The models for transect-level data are more complex because they estimate a random intercept of site
# 2) saves model output and calculates yield gaps due to pollinators
# 3) combines output from all models and creates a summary for analysis (lim_out)

#Note: because Bayesian analyses run random processes and are contingent on the number of iterations used, results may vary from our uploaded lim_out.csv file

##########################################################################################
##########################################################################################
# dataset S1 (98 field-level studies)
##########################################################################################

# empty dataframe to hold results
lim_out_1 = data.frame()

# list of studies in the dataset
studylist = unique(field_data_S1$study_id)
length(studylist) #98

# loop through studies, and run the three models for each
for (i in 1:length(studylist)) {
	  
	thisstudy = subset(field_data_S1, study_id==studylist[i])# 

	#############################################
	# segmented model, no random effects

	stanDat <- list(
		yield_z = thisstudy$yield_z, 
		all_insects_z = thisstudy$all_insects_z,
		N = nrow(thisstudy),
		bp_lower = quantile(thisstudy$all_insects_z, .05),
		bp_upper = quantile(thisstudy$all_insects_z, .95)
	)

	mod_seg1 <- stan(file = "pollen lim segmented non-mixed.stan", data = stanDat, iter = this_iterations, chains = 4)
	mod_seg1

	#############################################
	# linear model, no random effects

	stanDat <- list(
		yield_z = thisstudy$yield_z, 
		all_insects_z = thisstudy$all_insects_z,
		N = nrow(thisstudy)
	)

	mod_linear1 <- stan(file = "pollen lim linear non-mixed.stan", data = stanDat, iter = this_iterations, chains = 4)
	mod_linear1

	#############################################
	# flat (intercept-only) model

	stanDat <- list(
		yield_z = thisstudy$yield_z, 
		N = nrow(thisstudy)
	)

	mod_flat1 <- stan(file = "pollen lim flat non-mixed.stan", data = stanDat, iter = this_iterations, chains = 4)
	mod_flat1

	#############################################
	#############################################
	# yield gap calculations

	# 10 and 90% quantiles for insects and yield to define pollinator gap and yield gap
	this_pollinator_q10 = quantile(thisstudy$all_insects_z, .1)
	this_pollinator_q90 = quantile(thisstudy$all_insects_z, .9)
	this_yield_q10 = quantile(thisstudy$yield_z, .1)
	this_yield_q90 = quantile(thisstudy$yield_z, .9)
	this_yield_gap = this_yield_q90 - this_yield_q10
	this_pollinator_gap = this_pollinator_q90 - this_pollinator_q10
	
	#############################################
	# yield gap calculations for linear model
	
	# amount of yield gap due to pollinators
	# plug in pollinator q10 and q90 values in to yield=b1+b2*pollinators to get corresponding yield value 
	this_yield_gap_q10_intersection = summary(mod_linear1)$summary[1,1] + summary(mod_linear1)$summary[2,1]*this_pollinator_q10 
	this_yield_gap_q90_intersection = summary(mod_linear1)$summary[1,1] + summary(mod_linear1)$summary[2,1]*this_pollinator_q90
	this_yield_gap_closable_by_pollinators = this_yield_gap_q90_intersection - this_yield_gap_q10_intersection
	# percentage of yield gap due to pollinators
	this_percent_gap_closable_by_pollinators_linear = as.numeric(this_yield_gap_closable_by_pollinators/this_yield_gap)		
	
	# confidence intervals for yield gap due to pollinators
	this_yield_gap_closable_by_pollinators_lower = summary(mod_linear1)$summary[2,4]*this_pollinator_gap
	this_yield_gap_closable_by_pollinators_upper = summary(mod_linear1)$summary[2,8]*this_pollinator_gap

	this_percent_gap_closable_by_pollinators_linear_lower = as.numeric(this_yield_gap_closable_by_pollinators_lower/this_yield_gap)
	this_percent_gap_closable_by_pollinators_linear_upper = as.numeric(this_yield_gap_closable_by_pollinators_upper/this_yield_gap)

	#############################################
	# yield gap calculations for segmented model

	# calculate yield values (y) across the range of possible pollinator values (x) and save to a dataframe
	xvals1=seq(-5,summary(mod_seg1)$summary[3,1],.01)
	xvals2=seq(summary(mod_seg1)$summary[3,1],5,.01)
	seg_points = data.frame(
		x = c(xvals1, xvals2), 
		y=c(
			summary(mod_seg1)$summary[1,1] + summary(mod_seg1)$summary[2,1]*(xvals1-summary(mod_seg1)$summary[3,1]), 
			rep(summary(mod_seg1)$summary[1,1], length(xvals2))
		)
	)
	seg_points$x = round(seg_points$x, 2)

	# amount of yield gap due to pollinators
	this_yield_gap_q10_intersection = seg_points[which(seg_points$x==round(this_pollinator_q10,2)),]$y
	this_yield_gap_q90_intersection = seg_points[which(seg_points$x==round(this_pollinator_q90,2)),]$y
	this_yield_gap_closable_by_pollinators = this_yield_gap_q90_intersection - this_yield_gap_q10_intersection
	# percentage of yield gap due to pollinators
	this_percent_gap_closable_by_pollinators_seg = as.numeric(this_yield_gap_closable_by_pollinators/this_yield_gap)	
	
	# pollinator gap covered by the linear part of the segmented relationship, which is where all the gap-closing occurs
	this_pollinator_gap_to_breakpoint = summary(mod_seg1)$summary[3,1] - this_pollinator_q10

	# confidence intervals for yield gap due to pollinators
	# (based on CI for slope of line)
	this_yield_gap_closable_by_pollinators_lower = summary(mod_linear1, probs = c(0.25, .5, 0.75))$summary[2,4]*this_pollinator_gap_to_breakpoint
	this_yield_gap_closable_by_pollinators_upper = summary(mod_linear1, probs = c(0.25, .5, 0.75))$summary[2,6]*this_pollinator_gap_to_breakpoint

	this_percent_gap_closable_by_pollinators_seg_lower = as.numeric(this_yield_gap_closable_by_pollinators_lower/this_yield_gap)
	this_percent_gap_closable_by_pollinators_seg_upper = as.numeric(this_yield_gap_closable_by_pollinators_upper/this_yield_gap)

	#############################################
	#############################################
	# save output
	
	thisout = data.frame(
		study = studylist[i], 
		LOOIC_flat = loo(mod_flat1)[1]$estimates[3], 
		LOOIC_lin = loo(mod_linear1)[1]$estimates[3], 
		LOOIC_seg = loo(mod_seg1, pars = "log_lik")[1]$estimates[3], 
		breakpt = summary(mod_seg1)$summary[3,1], 
		perc_lim = nrow(subset(thisstudy, all_insects_z >= summary(mod_seg1)$summary[3,1]))/nrow(thisstudy),
		intercept_lin = summary(mod_linear1)$summary[1,1], 
		intercept_seg = summary(mod_seg1)$summary[1,1], 
		intercept_flat = summary(mod_flat1)$summary[1,1], 
		slope_lin = summary(mod_linear1)$summary[2,1], 
		slope_seg = summary(mod_seg1)$summary[2,1], 
		n = nrow(thisstudy), 
		yield_gap = this_yield_gap, 
		pollinator_gap = this_pollinator_gap, 
		gap_close_linear = this_percent_gap_closable_by_pollinators_linear, 
		gap_close_seg = this_percent_gap_closable_by_pollinators_seg,
		range1_linear = this_percent_gap_closable_by_pollinators_linear_upper,
		range2_linear = this_percent_gap_closable_by_pollinators_linear_lower,
		range1_seg = this_percent_gap_closable_by_pollinators_seg_upper,
		range2_seg = this_percent_gap_closable_by_pollinators_seg_lower,
		iterations = this_iterations
	)
	lim_out_1 = rbind(thisout, lim_out_1)
}

lim_out_1 


##########################################################################################
##########################################################################################
# dataset S2 (22 transect-level studies)
##########################################################################################

# empty dataframe to hold results
lim_out_2 = data.frame()

# list of studies in the dataset
studylist = unique(field_data_S2$study_id)
length(studylist) #98

# loop through studies, and run the three models for each
for (i in 1:length(studylist)) {
	  
	thisstudy = subset(field_data_S2, study_id==studylist[i])# 
	thisstudy$site_int = as.integer(as.factor(thisstudy$site_id))

	#############################################
	# segmented model, random intercepts of site (fixed slope and breakpoint)

	stanDat <- list(
		yield_z = thisstudy$yield_z, 
		all_insects_z = thisstudy$all_insects_z,
		site = thisstudy$site_int,
		N = nrow(thisstudy),
		J = length(unique(thisstudy$site_int)),
		bp_lower = quantile(thisstudy$all_insects_z, .05),
		bp_upper = quantile(thisstudy$all_insects_z, .95)
	)

	mod_seg2 <- stan(file = "pollen lim segmented mixed.stan", data = stanDat, iter = this_iterations, chains = 4)
	mod_seg2

	#############################################
	# linear model, random intercepts of site (fixed slope)

	stanDat <- list(
		yield_z = thisstudy$yield_z, 
		all_insects_z = thisstudy$all_insects_z,
		site = thisstudy$site_int,
		N = nrow(thisstudy),
		J = length(unique(thisstudy$site_int))
	)

	mod_linear2 <- stan(file = "pollen lim linear mixed.stan", data = stanDat, iter = this_iterations, chains = 4)
	mod_linear2

	#############################################
	# flat (intercept-only) model, random intercepts of site

	stanDat <- list(
		yield_z = thisstudy$yield_z, 
		site = thisstudy$site_int,
		N = nrow(thisstudy),
		J = length(unique(thisstudy$site_int))
	)

	mod_flat2 <- stan(file = "pollen lim flat mixed.stan", data = stanDat, iter = this_iterations, chains = 4)
	mod_flat2

	#############################################
	#############################################
	# yield gap calculations

	# 10 and 90% quantiles for insects and yield to define pollinator gap and yield gap
	this_pollinator_q10 = quantile(thisstudy$all_insects_z, .1)
	this_pollinator_q90 = quantile(thisstudy$all_insects_z, .9)
	this_yield_q10 = quantile(thisstudy$yield_z, .1)
	this_yield_q90 = quantile(thisstudy$yield_z, .9)
	this_yield_gap = this_yield_q90 - this_yield_q10
	this_pollinator_gap = this_pollinator_q90 - this_pollinator_q10
	
	#############################################
	# yield gap calculations for linear model
	
	# amount of yield gap due to pollinators
	# plug in pollinator q10 and q90 values in to yield=b1+b2*pollinators to get corresponding yield value 
	this_yield_gap_q10_intersection = summary(mod_linear2)$summary[1,1] + summary(mod_linear2)$summary[2,1]*this_pollinator_q10 
	this_yield_gap_q90_intersection = summary(mod_linear2)$summary[1,1] + summary(mod_linear2)$summary[2,1]*this_pollinator_q90
	this_yield_gap_closable_by_pollinators = this_yield_gap_q90_intersection - this_yield_gap_q10_intersection
	# percentage of yield gap due to pollinators
	this_percent_gap_closable_by_pollinators_linear = as.numeric(this_yield_gap_closable_by_pollinators/this_yield_gap)		
	
	# confidence intervals for yield gap due to pollinators
	this_yield_gap_closable_by_pollinators_lower = summary(mod_linear2)$summary[2,4]*this_pollinator_gap
	this_yield_gap_closable_by_pollinators_upper = summary(mod_linear2)$summary[2,8]*this_pollinator_gap

	this_percent_gap_closable_by_pollinators_linear_lower = as.numeric(this_yield_gap_closable_by_pollinators_lower/this_yield_gap)
	this_percent_gap_closable_by_pollinators_linear_upper = as.numeric(this_yield_gap_closable_by_pollinators_upper/this_yield_gap)

	#############################################
	# yield gap calculations for segmented model

	# calculate yield values (y) across the range of possible pollinator values (x) and save to a dataframe
	xvals1=seq(-5,summary(mod_seg2)$summary[3,1],.01)
	xvals2=seq(summary(mod_seg2)$summary[3,1],5,.01)
	seg_points = data.frame(
		x = c(xvals1, xvals2), 
		y=c(
			summary(mod_seg2)$summary[1,1] + summary(mod_seg2)$summary[2,1]*(xvals1-summary(mod_seg2)$summary[3,1]), 
			rep(summary(mod_seg2)$summary[1,1], length(xvals2))
		)
	)
	seg_points$x = round(seg_points$x, 2)

	# amount of yield gap due to pollinators
	this_yield_gap_q10_intersection = seg_points[which(seg_points$x==round(this_pollinator_q10,2)),]$y
	this_yield_gap_q90_intersection = seg_points[which(seg_points$x==round(this_pollinator_q90,2)),]$y
	this_yield_gap_closable_by_pollinators = this_yield_gap_q90_intersection - this_yield_gap_q10_intersection
	# percentage of yield gap due to pollinators
	this_percent_gap_closable_by_pollinators_seg = as.numeric(this_yield_gap_closable_by_pollinators/this_yield_gap)	
	
	# pollinator gap covered by the linear part of the segmented relationship, which is where all the gap-closing occurs
	this_pollinator_gap_to_breakpoint = summary(mod_seg2)$summary[3,1] - this_pollinator_q10

	# confidence intervals for yield gap due to pollinators
	# (based on CI for slope of line)
	this_yield_gap_closable_by_pollinators_lower = summary(mod_linear2, probs = c(0.25, .5, 0.75))$summary[2,4]*this_pollinator_gap_to_breakpoint
	this_yield_gap_closable_by_pollinators_upper = summary(mod_linear2, probs = c(0.25, .5, 0.75))$summary[2,6]*this_pollinator_gap_to_breakpoint

	this_percent_gap_closable_by_pollinators_seg_lower = as.numeric(this_yield_gap_closable_by_pollinators_lower/this_yield_gap)
	this_percent_gap_closable_by_pollinators_seg_upper = as.numeric(this_yield_gap_closable_by_pollinators_upper/this_yield_gap)

	#############################################
	#############################################
	# save output
	
	thisout = data.frame(
		study = studylist[i], 
		LOOIC_flat = loo(mod_flat2)[1]$estimates[3], 
		LOOIC_lin = loo(mod_linear2)[1]$estimates[3], 
		LOOIC_seg = loo(mod_seg2, pars = "log_lik")[1]$estimates[3], 
		breakpt = summary(mod_seg2)$summary[3,1], 
		perc_lim = nrow(subset(thisstudy, all_insects_z >= summary(mod_seg1)$summary[3,1]))/nrow(thisstudy),
		intercept_lin = summary(mod_linear2)$summary[1,1], 
		intercept_seg = summary(mod_seg2)$summary[1,1], 
		intercept_flat = summary(mod_flat2)$summary[1,1], 
		slope_lin = summary(mod_linear2)$summary[2,1], 
		slope_seg = summary(mod_seg2)$summary[2,1], 
		n = nrow(thisstudy), 
		yield_gap = this_yield_gap, 
		pollinator_gap = this_pollinator_gap, 
		gap_close_linear = this_percent_gap_closable_by_pollinators_linear, 
		gap_close_seg = this_percent_gap_closable_by_pollinators_seg,
		range1_linear = this_percent_gap_closable_by_pollinators_linear_upper,
		range2_linear = this_percent_gap_closable_by_pollinators_linear_lower,
		range1_seg = this_percent_gap_closable_by_pollinators_seg_upper,
		range2_seg = this_percent_gap_closable_by_pollinators_seg_lower,
		iterations = this_iterations
	)
	lim_out_2 = rbind(thisout, lim_out_2)
}

lim_out_2


##########################################################################################
##########################################################################################
# combine results and summarize
##########################################################################################

lim_out = rbind(lim_out_1, lim_out_2)

#############################################
# pick the best relationship based on LOOIC and whether it is negative or positive

lim_out$decision = ""	# just a placeholder that will be overwritten

for (i in 1:nrow(lim_out)) {
  if ((lim_out$LOOIC_seg[i] < lim_out$LOOIC_flat[i]) & (lim_out$LOOIC_seg[i] < lim_out$LOOIC_lin[i])) {
    if (lim_out$slope_seg[i] >0) {
      lim_out$decision[i] <- "segmented"
    } else {
      lim_out$decision[i] <- "negative segmented"
    }
  }
  if ((lim_out$LOOIC_lin[i] < lim_out$LOOIC_flat[i]) & (lim_out$LOOIC_lin[i] < lim_out$LOOIC_seg[i])) {
    if (lim_out$slope_lin[i] >0) {
      lim_out$decision[i] <- "linear"
    } else {
      lim_out$decision[i] <- "negative linear"
    }
  }	
  if ((lim_out$LOOIC_flat[i] < lim_out$LOOIC_seg[i]) & (lim_out$LOOIC_flat[i] < lim_out$LOOIC_lin[i])) {
    lim_out$decision[i] <- "0"
  }		
}
lim_out


#############################################
# create summary variables for the fraction of fields limited (perc_lim2) and percent of yield gap due to pollinators (gap_perc)
# note here we set both metrics to 0 when the flat or negative models were selected

lim_out$perc_lim2 = 50 # just a placeholder that will be overwritten
lim_out$gap_perc = 0 # just a placeholder that will be overwritten

for (i in 1:nrow(lim_out)) {
  if (lim_out$decision[i] =="linear") {
    lim_out$perc_lim2[i] <- 1		# for linear we assume all fields are limited
	lim_out$gap_perc[i] <- lim_out$gap_close_linear[i]
  }
  if (lim_out$decision[i] =="0") {
    lim_out$perc_lim2[i] <- 0		# for flat we assume no fields are limited
	lim_out$gap_perc[i] <- 0		# for flat we assume 0% of yield gap is due to pollinators
  }	
  if (lim_out$decision[i] =="segmented") {
    lim_out$perc_lim2[i] <- lim_out$perc_lim[i]		# for segmented we assume the fraction of fields left of the breakpoint are limited
	lim_out$gap_perc[i] <- lim_out$gap_close_seg[i]
  }	
  if (lim_out$decision[i] =="negative linear") {
    lim_out$perc_lim2[i] <- 0		# for negative we assume no fields are limited
	lim_out$gap_perc[i] <- 0		# for negative we assume 0% of yield gap is due to pollinators
  }	
  if (lim_out$decision[i] =="negative segmented") {
    lim_out$perc_lim2[i] <- 0		# for negative we assume no fields are limited
	lim_out$gap_perc[i] <- 0		# for negative we assume 0% of yield gap is due to pollinators
  }	  
}

lim_out

